///////////////////////////////////////////////
//
// A L-System fractal generator
// Copyright (c) 2005 Jari Komppa
//
//
///////////////////////////////////////////////
// License
///////////////////////////////////////////////
// 
//     This software is provided 'as-is', without any express or implied
//     warranty.    In no event will the authors be held liable for any damages
//     arising from the use of this software.
// 
//     Permission is granted to anyone to use this software for any purpose,
//     including commercial applications, and to alter it and redistribute it
//     freely, subject to the following restrictions:
// 
//     1. The origin of this software must not be misrepresented; you must not
//        claim that you wrote the original software. If you use this software
//        in a product, an acknowledgment in the product documentation would be
//        appreciated but is not required.
//     2. Altered source versions must be plainly marked as such, and must not be
//        misrepresented as being the original software.
//     3. This notice may not be removed or altered from any source distribution.
// 
// (eg. same as ZLIB license)
// 
//
///////////////////////////////////////////////


enum STACKPUSH
{
    STACK_POS = 1,
    STACK_DIR = 2,
    STACK_LEN = 4,
    STACK_AXIS = 8,
    STACK_COLOR = 16,
    STACK_EVERYTHING = 0xff
};

class vertex
{
public:
	float mX, mY, mZ;
    vertex() {mX = mY = mZ = 0; }
};

class lsystem_stackitem
{
public:
    int mWhat;
    vertex mPos;
    vertex mDir;
    float mLen;
    int mAxis;
    int mColor;
};

class lsystem
{
public:
    lsystem_stackitem mStack[256];
    int mStackptr;
    float * mVtx[8];
    int mVtxcount[8];
    
    float mAngle;
    float mWave;
    float mLength;
    float mGenlength;
    int mStyle;
    int mAxiomcount;
    int mIterations;
    char ** mAxiom;
    int mColor[8];
    
    lsystem();
    ~lsystem();
    void generate(char *aBuffer);
    void parse(char * aTarget, char *aTemp, int aTargetlen, int aOutputDebugLog = 0);
    void draw();
    void push(int aWhat, vertex &aPos, vertex &aDir, float aLen, int aAxis, int aColor);
    int pop(vertex &aPos, vertex &aDir, float &aLen, int &aAxis, int &aColor);
    float tick();
    static lsystem * load_scene(char *aFilename);
    

};
