#ifdef _MSC_VER
#include <windows.h> // needed to get GL stuff to work
#endif
#include <SDL.h>
#include <gl/gl.h>
#include <gl/glu.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "lsystem.h"

lsystem *lsys;

void setupcamera()
{  
    float angle = SDL_GetTicks() / 10000.0f;
  
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    glMatrixMode( GL_MODELVIEW );
    glLoadIdentity( );

    glTranslatef( 0.0, -4.0, -20.0 );

    glRotatef( angle * 360, 0, 1.0f, 0);
}


static void draw_screen()
{
    setupcamera();
    lsys->draw();
    SDL_GL_SwapBuffers();
}


static void setup_opengl( int width, int height )
{
    float ratio = (float) width / (float) height;

    glShadeModel( GL_SMOOTH );

    glCullFace( GL_BACK );
    glFrontFace( GL_CCW );
    glEnable( GL_CULL_FACE );

    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LEQUAL);

    glClearColor( 0, 0, 0, 0 );

    glViewport( 0, 0, width, height );

    glMatrixMode( GL_PROJECTION );
    glLoadIdentity( );
    gluPerspective( 60.0, ratio, 1.0, 1024.0 );
}



int main (int argc, char** args)
{
    if (argc > 1)
        lsys = lsystem::load_scene(args[1]);
	else
	{
		fprintf(stderr,"Usage: lsys <lsystem definition file>\n");
		return -1;
	}
    
    const SDL_VideoInfo *info = NULL;
    int width = 0;
    int height = 0;
    int bpp = 0;
    int flags = 0;

    if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_NOPARACHUTE) < 0) 
    {
        fprintf(stderr, "Video initialization failed: %s\n", SDL_GetError());
        SDL_Quit();
        exit(0);
    }

    info = SDL_GetVideoInfo();

    if (!info) 
    {
        fprintf(stderr, "Video query failed: %s\n", SDL_GetError());
        SDL_Quit();
        exit(0);
    }

    width = 640;
    height = 480;
    bpp = info->vfmt->BitsPerPixel;

    SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 5);
    SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 5);
    SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 5);
    SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 16);
    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);

    flags = SDL_OPENGL;// | SDL_FULLSCREEN;

    if (SDL_SetVideoMode(width, height, bpp, flags) == 0) 
    {
        fprintf( stderr, "Video mode set failed: %s\n", SDL_GetError());
        SDL_Quit();
        exit(0);
    }

    setup_opengl(width, height);

    while (1) 
    {
        SDL_Event event;
    
        while (SDL_PollEvent(&event)) 
        {
            switch (event.type) 
            {
            case SDL_KEYDOWN:
            {
                int k = event.key.keysym.sym;

                switch (k)
                {
                case SDLK_ESCAPE:
                    SDL_Quit();
                    exit(0);
                    break;
                case SDLK_SPACE:
                    delete lsys;
                    lsys = lsystem::load_scene(args[1]);
                    break;                        
                }
            }
                break;
            case SDL_QUIT:
                SDL_Quit();
                exit(0);
                break;
            }
        }
        draw_screen();
        SDL_Delay(20);
    }

    return 0;
}
